% Code for the JMD 2009 product design with price competition paper 
% Case study 1: CDH Painkiller product positioning problem
% Solve using MPEC Nash strategy with using Ralph and Wright (2004)'s
%   Reg(t) regularization scheme
% Updated: 7th March, 2009

clear all
clc
format compact

global I J N chi v0
global Z y vv b w t
global p_init p_solu Z z_L

load pzsol_nash_true % load true solution - pzsol_nash_true
TolOverall=2e-15; % best: 1e-16
%I subjects or consumers (30 individuals),
%J brand names
%N coordinates to describe brands /asp, asub, caff, aing/;
% new product is J=15 (14 competitiors)
I=30; J=15; N=4;

% the tolerance in the Ralph and Wright's Reg(t) formulation
t=1e-10;

%chi randomness increases as chi decreases   / 3 /,
%Kdemand demand for no drug,
%v0=K  constant term in for no purchase option  / 1 /;
chi=3; v0=1;

%x(J,N) brand J's amount of N,
%y(I,N) subject I's preference for N,
%vv(I)  subject I's importance weight for every N,
%w(I)   price attributes,
%b(I)   constant term in DU eqn,
%C(J)   average cost of production,
%p_init(J) initial prices for brand J,
%p_solu(J) solution from Choi-DeSarbo-Harker paper,
%demand(J) computed demand for brand J,
% DU(I,J) matrix in disutility function;

% get data in choi.dat
init_choi_data;
% Z unit: 1000mg = g
C=sum(Z(:,1)*10*0.3+Z(:,2)*10*0.8+Z(:,3)*10*0.4+Z(:,4)*10*0.2,2); 
brand=['A  ';'B  ';'C  ';'D  ';'E  ';...
       'F  ';'G  ';'H  ';'I  ';'J  ';...
       'K  ';'L  ';'M  ';'N  ';'New'];
p_L=0; p_U=30;
z_L=0; z_U=1;

% pz size=59 - here is the data structure
% p=pz(1:15);     % #15 is the price of the new product (14 competitors) (size=15)
% znew=pz(16:19); % the new product attributes (size=4)
% muPU=pz(20:34); % Lagrange multipliers for price upper bound (size=15)
% muPL=pz(35:49); % Lagrange multipliers for price lower bound (size=15)
% muZU=pz(50:53); % Lagrange multipliers for new product z up_bound (size=4)
% muZL=pz(54:57); % Lagrange multipliers for new product z lo_bound (size=4)   
% muG1=pz(58);    % Lagrange multiplier  for g1 (size=1)
% muG2=pz(59);    % Lagrange multiplier  for g2 (size=1)
   
% lower and upper bounds
lb=[C; 0;
    zeros(44,1)]; % all lower bounds
ub=[3*C; 10; ... % price upper bound (size=15)
    ones(4,1)*.5;...   % design z upper bound (size=4)
    ones(40,1)*100];   % Lagrange multiplier upper bounds (size=40) 

% start point
%     p_j' p_new  z1  z2  z3  z4
pz0=[C+0.1; 2; 0;  0;  0;  0;
     zeros(40,1); ]; %40 Lagrange multipliers
pz0(8)=1.99; % p(8) is fixed at $1.99
p_init(15)=3.74;  

options = optimset('Display','iter','Algorithm','active-set', ...
    'GradObj','off','GradConstr','off', ...
    'TolCon',TolOverall,'TolX',TolOverall,'TolFun',TolOverall);
tic;
[pzsol,fval,exitflag]=fmincon(@objfun,pz0,[],[],[],[],lb,ub,@focfun,options);
% [pzsol,fval,exitflag]=ktrlink(@objfun,pz0,[],[],[],[],lb,ub,...
%            @focfun,options);
cpu_time=toc;

p=pzsol(1:15);     % #15 is the price of the new product
pnew=pzsol(15);
znew=pzsol(16:19); % the new product attributes
z=[Z;znew'];
c=sum(z(:,1)*3+z(:,2)*8+z(:,3)*4+z(:,4)*2,2);

s=gets(p,z);
demand=sum(s,1)';
mktshr=sum(s,1)'/I;
profit=(p-c).*demand;

disp('brand  $ cost  p_init  $price  %share  demand  profit');
disp('-----  ------  ------  ------  ------  ------  ------');
for j=1:J
  disp(sprintf('%2d%3s  %6.3f  %6.3f  %6.3f  %6.3f  %6.3f  %6.3f',...
    j,brand(j,1:3),c(j),p_init(j),p(j),mktshr(j)*100,demand(j),profit(j)));
end
disp('-----------------------------------------------------');
disp('The solution for Nash game scenario:');
disp(sprintf('Optimal price of new product (#15) = $%6.3f',pnew));
disp(sprintf('Aspirin  = %7.3f mg',znew(1)*1e3));
disp(sprintf('Asp-sub  = %7.3f mg',znew(2)*1e3));
disp(sprintf('Caffeine = %7.3f mg',znew(3)*1e3));
disp(sprintf('Add ingd = %7.3f mg',znew(4)*1e3));


options = optimset('Display','off','Algorithm','active-set','TolCon',TolOverall,'TolX',TolOverall);
% Verify Nash of each product
disp('=== start to do Nash check for each product ===');
for j=1:(J-1) % verify all competing products (except J: new product)
    pj0=c(j)+1;
    [pjchk,nfval,exitflag]=fmincon(@objchk,pj0,[],[],[],[],c(j),3*c(j),[],options,pzsol,j);
    profitchk_j=-1*nfval;
    %disp([profitchk_j, profit(j)]);
    if round(profitchk_j*1e6) > round(profit(j)*1e6)
        disp(sprintf('Nash check for j=%2d%-3s failed (%-18.16f >  %-18.16f)',...
            j,brand(j,1:3),profitchk_j,profit(j)));
    else
        disp(sprintf('Nash check for j=%2d%-3s passed (%-18.16f <= %-18.16f)',...
            j,brand(j,1:3),profitchk_j,profit(j)));
    end
end

% normaliz first and calculate solution error
pz_max=[10*ones(15,1); .5*ones(4,1)];
pz_min=zeros(19,1);
norm_inf=max( abs(pzsol(1:19)-pzsol_nash_true)./(pz_max-pz_min) );
norm_euc=sqrt(sum(( (pzsol(1:19)-pzsol_nash_true)./(pz_max-pz_min)).^2));
disp(sprintf('norm_inf = %-0.15f',norm_inf));
disp(sprintf('norm_euc = %-0.15f',norm_euc));
disp(sprintf('CPU time = %-0.3f',cpu_time));