% solve Nash price equilibrium using SIO method
% in order to compare CPU time with the proposed method in paper

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% In this problem, a Nash price equilibrium is sought.
% Different brand names of analgesics are available, with
% different levels of aspirin, aspirin substitutes, caffeine,
% and added ingredients.  Demand is determined by the consumers'
% disutility function, so that the demand for brand I is the
% sum over J of the probability that consumer J will buy brand I.
% Each brand name tries to maximize profit
% by setting its price at an optimal level (computed assuming
% all other prices constant).
% Reference:
% S. Chan Choi,Wayne S. DeSarbo, and Patrick T. Harker,
% "Product Positioning Under Price Competition", Management Science
% (36 no. 2) 1990, pp 175-199.
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all
clc
format compact

global I J N chi v0
global Z y vv b w C
global p_init p_solu Z
load pzsol_nash_true.mat % read the true solution pzsol_nash_true
TolOverall=5e-9;

%I   subjects or consumers / 1 % 30 /,
%J   brand names 1
%N   coordinates to describe brands / asp, asub, caff, aing /;
% new product is J=15
I=30; J=15; N=4;

%chi randomness increases as chi decreases   / 3 /,
%Kdemand demand for no drug,
%v0=K  constant term in for no purchase option  / 1 /;
chi=3; v0=1;

%x(J,N) brand J's amount of N,
%y(I,N) subject I's preference for N,
%vv(I)  subject I's importance weight for every N,
%w(I)   price attributes,
%b(I)   constant term in DU eqn,
%C(J)   average cost of production,
%p_init(J) initial prices for brand J,
%p_solu(J) solution from Choi-DeSarbo-Harker paper,
%demand(J) computed demand for brand J,
% DU(I,J) matrix in disutility function;

% get data in choi.dat
init_choi_data;
C=sum(Z(:,1)*10*0.3+Z(:,2)*10*0.8+Z(:,3)*10*0.4+Z(:,4)*10*0.2,2);
brand=['A  ';'B  ';'C  ';'D  ';'E  ';...
       'F  ';'G  ';'H  ';'I  ';'J  ';...
       'K  ';'L  ';'M  ';'N  ';'New'];
   
% lower and upper bounds
% comp_p newp  z1 z2 z3 z4
lb=[ 0; 0; 0; 0; 0;]; 
ub=[12; 1; 1; 1; 1;];
% start point
pznew0=[3.74; .1;.1;.1;.1];
pz_update=[C+0.5; pznew0];

%p0=[p_init];
%p0=p_solu;
p_init(15)=3.74;

options = optimset('Display','off','Algorithm','active-set','TolCon',TolOverall,'TolX',TolOverall);


% run SIO to solve the Nash problem
% each competitor can change their price only
% but in New Product's turn, it can change price and design

covtolflag=0;
pz_old=0;
count=1;
disp(' A    B    C    D    E    F    G    H    I    J    K    L    M    N   pnew  z1     z2     z3     z4');

tic;
% big SIO loop
while(covtolflag ~=(J+N)) % all prices and new design attributes are converged
    count=count+1;
    for j=1:J-1 % loop for all competitors' profit maximization on pricing
        p0=1;
        if j==8
            pz_update(8)=1.99; % generic product H with fixed price $1.99
        else
            [pj,profitj,exitflag]=fmincon(@profitjfun,C(j)+1,[],[],[],[],C(j),3*C(j),[],options,pz_update,j);
            if exitflag < 1
                disp('warning: competitor pricing optimization exitflag < 1');
            end
            pz_update(j)=pj; % save the solution for product j
        end
%        profit(j)=-1*profitj;
%        disp(profit(j));
    end % end of competitor's price decision loop
    
    % now run optimization for new product's both price and design
    j=J;
    [pznewsol,fval,exitflag]=fmincon(@profitnpfun,pznew0,[],[],[],[],lb,ub,@confun,options,pz_update);
    if exitflag < 1
        disp('warning: new product optimization exitflag < 1');
    end
    pnew=pznewsol(1);   % the new product price solution
    znew=pznewsol(2:5); % the new product attribute solution
    % assemble price and design variables into a vector
    pz_update(J:J+N)=[pnew;znew];  % renew pz_update
    
    disp(sprintf('%-0.2f %-0.2f %-0.2f %-0.2f %-0.2f %-0.2f %-0.2f %-0.2f %-0.2f %-0.2f %-0.2f %-0.2f %-0.2f %-0.2f %-0.2f %0.3f  %0.3f  %0.3f  %0.3f',...
         pz_update(1),pz_update(2),pz_update(3),pz_update(4),pz_update(5),pz_update(6),pz_update(7),...
         pz_update(8),pz_update(9),pz_update(10),pz_update(11),pz_update(12),pz_update(13),pz_update(14),...
         pnew,znew(1),znew(2),znew(3),znew(4)));
    
    covtol=abs(pz_update - pz_old);
    covtolflag = sum(covtol < TolOverall*10);
    pz_old=pz_update;
    
end % end of SIO loop
cpu_time=toc;;

p=pz_update(1:15);     % #15 is the price of the new product
pnew=pz_update(15);
znew=pz_update(16:19); % the new product attributes
z=[Z;znew'];
c=sum(z(:,1)*10*0.3+z(:,2)*10*0.8+z(:,3)*10*0.4+z(:,4)*10*0.2,2);

s=gets(p,z);
demand=sum(s,1)';
mktshr=sum(s,1)'/I;
profit=(p-c).*demand;

disp('brand  $ cost  p_init  $price  %share  demand  profit');
disp('-----  ------  ------  ------  ------  ------  ------');
for j=1:J
  disp(sprintf('%2d%3s  %6.3f  %6.3f  %6.3f  %6.3f  %6.3f  %6.3f',...
    j,brand(j,1:3),c(j),p_init(j),p(j),mktshr(j)*100,demand(j),profit(j)));
end
disp('-----------------------------------------------------');
disp('The solution for Nash game scenario:');
disp(sprintf('Optimal price of new product (#15) = $%6.3f',pnew));
disp(sprintf('Aspirin  = %7.3f mg',znew(1)*1e3));
disp(sprintf('Asp-sub  = %7.3f mg',znew(2)*1e3));
disp(sprintf('Caffeine = %7.3f mg',znew(3)*1e3));
disp(sprintf('Add ingd = %7.3f mg',znew(4)*1e3));

% Verify Nash of each product
disp('=== start to do Nash check for each product ===');
for j=1:(J-1) % verify all competing products (except J: new product)
    pj0=c(j);
    [pjchk,nfval,exitflag]=fmincon(@objchk,pj0,[],[],[],[],c(j),3*c(j),[],options,pz_update,j);
    profitchk_j=-1*nfval;
    %disp([profitchk_j, profit(j)]);
    if round(profitchk_j*1e3) > round(profit(j)*1e3)
        disp(sprintf('Nash check for j=%2d%-3s failed (%-12.10f >  %-12.10f)',...
            j,brand(j,1:3),profitchk_j,profit(j)));
    else
        disp(sprintf('Nash check for j=%2d%-3s passed (%-12.10f <= %-12.10f)',...
            j,brand(j,1:3),profitchk_j,profit(j)));
    end
end

pzsol=pz_update;
pz_max=[10*ones(15,1); .5*ones(4,1)];
pz_min=zeros(19,1);
% normaliz first and calculate solution error
norm_inf=max( abs((pzsol-pzsol_nash_true)./(pz_max-pz_min)) );
norm_euc=sqrt(sum(((pzsol-pzsol_nash_true)./(pz_max-pz_min)).^2));
disp(sprintf('norm_inf = %-0.15f',norm_inf));
disp(sprintf('norm_euc = %-0.15f',norm_euc));
disp(sprintf('CPU time = %-0.3f with %-3d iterations.\n',cpu_time,count-1));