% STKG-RELAXATION-PARALLEL
% solve Nash price equilibrium using CDH/VI-Harker method
% in order to compare CPU time with the proposed method in paper

clear all
clc
format compact

global I J N chi v0
global Z y vv b w C
global p_init p_solu Z
global pfollower COVGETOL

load pzsol_stkg_true.mat

TolOverall=1e-5;
COVGETOL=TolOverall;
%I   subjects or consumers / 1 % 30 /,
%J   brand names 1
%N   coordinates to describe brands / asp, asub, caff, aing /;
% new product is J=15
I=30; J=15; N=4;

%chi randomness increases as chi decreases   / 3 /,
%Kdemand demand for no drug,
%v0=K  constant term in for no purchase option  / 1 /;
chi=3; v0=1;

%x(J,N) brand J's amount of N,
%y(I,N) subject I's preference for N,
%vv(I)  subject I's importance weight for every N,
%w(I)   price attributes,
%b(I)   constant term in DU eqn,
%C(J)   average cost of production,
%p_init(J) initial prices for brand J,
%p_solu(J) solution from Choi-DeSarbo-Harker paper,
%demand(J) computed demand for brand J,
% DU(I,J) matrix in disutility function;

% get data in choi.dat
init_choi_data;
C=sum(Z(:,1)*10*0.3+Z(:,2)*10*0.8+Z(:,3)*10*0.4+Z(:,4)*10*0.2,2);
brand=['A';'B';'C';'D';'E';'F';'G';'H';'I';'J';'K';'L';'M';'N';'X'];
%pfollower=[6.99;3.97;5.29;3.29;2.69;3.89;5.31;1.99;5.75;4.99;7.59;4.99;3.69;4.99];
pfollower=C+0.1;
p_init(15)=3.74;

% lower and upper bounds
% newp  z1 z2 z3 z4
lb=[0; 0; 0; 0; 0;]; 
ub=[6; 1; 1; 1; 1;];
% start point
pznew0=[3.74; .124;.201;.0;.0];
options = optimset('Display','iter','Algorithm','active-set','TolCon',TolOverall,'TolX',TolOverall);

time0 = cputime;
% run CDH's algorithm to solve the Nash problem
% each competitor can change their price only
% but in New Product's turn, it can change price and design
% now run optimization for nwe product's price and design
[pznewsol,fval,exitflag]=fmincon(@profitnpfun,pznew0,[],[],[],[],lb,ub,@confun,options);
cpu_time=cputime-time0;

if exitflag < 1
    disp('warning: new product optimization exitflag < 1');
end

pnew=pznewsol(1);   % the new product price solution
znew=pznewsol(2:5); % the new product attribute solution
% assemble price and design variables into a vector
p=[pfollower; pnew];     
z=[Z;znew'];
pz_update=[p;znew];
c=sum(z(:,1)*10*0.3+z(:,2)*10*0.8+z(:,3)*10*0.4+z(:,4)*10*0.2,2);

s=gets(p,z);
demand=sum(s,1)';
mktshr=sum(s,1)'/I;
profit=(p-c).*demand;

disp('brand  $ cost  p_init  $price  %share  demand  profit');
disp('-----  ------  ------  ------  ------  ------  ------');
for j=1:J
  disp(sprintf('%2d %1s   %6.3f  %6.3f  %6.3f  %6.3f  %6.3f  %6.3f',...
    j,brand(j,1),c(j),p_init(j),p(j),mktshr(j)*100,demand(j),profit(j)));
end
disp('-----------------------------------------------------');
disp('The solution for Stackelberg scenario:');
disp(sprintf('Optimal price of new product (#15) = $%6.3f',pnew));
disp(sprintf('Aspirin  = %-0.3f mg',znew(1)*1e3));
disp(sprintf('Asp-sub  = %-0.3f mg',znew(2)*1e3));
disp(sprintf('Caffeine = %-0.3f mg',znew(3)*1e3));
disp(sprintf('Add ingd = %-0.3f mg',znew(4)*1e3));

% Verify Nash condition of each product
options = optimset('Display','off','Algorithm','active-set','TolCon',TolOverall,'TolX',TolOverall);
disp('=== start to do Nash check for each product ===');
for j=1:(J-1) % verify all competing products (except J: new product)
    pj0=c(j);
    [pjchk,nfval,exitflag]=fmincon(@objchk,pj0,[],[],[],[],c(j),3*c(j),[],options,pz_update,j);
    profitchk_j=-1*nfval;

%    if profitchk_j > profit(j)
    if round(profitchk_j*1e3) > round(profit(j)*1e3)
        disp(sprintf('Nash check for j=%2d%-3s failed (%-12.3f >  %-12.3f)',...
            j,brand(j,1),profitchk_j,profit(j)));
    else
        disp(sprintf('Nash check for j=%2d%-3s passed (%-12.3f <= %-12.3f)',...
            j,brand(j,1),profitchk_j,profit(j)));
    end
end

% normaliz first and calculate solution error
pzsol=pz_update;
pz_max=[10*ones(15,1); .5*ones(4,1)];
pz_min=zeros(19,1);
norm_inf=max( abs(pzsol(1:19)-pzsol_stkg_true)./(pz_max-pz_min) );
norm_euc=sqrt(sum(( (pzsol(1:19)-pzsol_stkg_true)./(pz_max-pz_min)).^2));
disp(sprintf('norm_inf = %-0.15f',norm_inf));
disp(sprintf('norm_euc = %-0.15f',norm_euc));
disp(sprintf('CPU time = %-0.3f',cpu_time));