$Title Angle Grinder Design under Stackelberg Price Competition

$oneolcom
$ontext
Filename: grinder_reg_stkg.gms
This code is for the case study in JMD paper (Stackelberg version)

Optimize new product under price competition with existing products
  in the market (a class II problem).

Source of demand model (Latent class model):
   Besharati, B., L. Luo, S. Azarm, and P. K. Kannan,
   "Multi-Objective Single Product Robust Optimization: An Integrated
    Design and Marketing Approach,"
   ASME Journal of Mechanical Design, Vol. 128, No. 4, pp. 884-892, 2006.
   Originally from Lan Lou's dissertation (2005)

Source of the estimated costs
   Lou, L., Kannan, P.K., and Ratchord, B.T., 2007,
   "New product development Under Channel Acceptance,"
   Marketing Science, 26(2), pp. 149-163.

This is a problem in MINLP form:
  1 Continuous variables: prices
  4 Discrete variables  : levels of amps, life, switch and girth

The follower's price equilibrium KKT conditions are regularized by
  using the RegComp(t) method presented in Ralph and Wright's 2004 paper.

Code by CS Norman Shiau
Design Decisions Laboratory
Mechanical Enigneering
Carnegie Mellon University
Pittbsurgh, PA, USA 15213
Latest update: March 11, 2009

$offtext

sets
  seg      market segment /seg1*seg4/,
  j        exisitng products in market /A,B,C/,
  atts     product attributes /amps,life,switch,girth/,
  amps     level of current ratings /6,9,12/,
  life     level of product life /80,110,150/,
  switch   type of swtich /PD,TS,SS,TG/,
  girth    type of girth /S,L/;
alias (j,jj);

parameters
  Q        total market size /9/,    !! 9 million (Lou, 2007)
  segratio(seg) ratio of market segment
          / seg1=0.378, seg2=0.248, seg3=0.121, seg4=0.253 /,
  mktsize(seg)  size of market segment,
  a2(seg) a2 coefficient of quardartic fitting funtion for price part-worth
          / seg1= 0.0020443, seg2=0.0026623,
            seg3=-0.0044753, seg4=0.0005643 /,
  a1(seg) a1 coefficient of quardartic fitting funtion for price part-worth
          / seg1=-0.4029413, seg2=-0.5270453,
            seg3= 0.8923093, seg4=-0.1119013 /,
  a0(seg) a1 coefficient of quardartic fitting funtion for price part-worth
          / seg1= 18.962681, seg2=24.929959,
            seg3=-42.556882, seg4= 5.303201 /,
  pw_no(seg) partworth of outside good
          / seg1=-0.022, seg2=-0.022, seg3=-0.022, seg4=-0.022 /,
  pw_bx(seg) partworth for brand X
          / seg1=-0.545, seg2=0.454, seg3=2.211, seg4=-0.165/,
*  c(j) unit cost of product /A=70.74,B=103.62,C=51.08/, !! cost data from Lou's thesis (2007)
  c(j)          cost /A=68.15,B=100.94,C=49.58/,
  cnew          cost of new product /75/,
  vz(seg,j)     utiliy (except price) of competing products,
  vcpt(seg,j)   utiliy (including price) of competing products,
  mktshr(seg,j) market shares of competing products,
  shrnew(seg)   market share of new product,
  shrno(seg)    market share of no-purchase option (outside good),
  profit(j)     profits of competing products,
  t             relaxation parameter in the RegComp(t) method /1e-8/;

mktsize(seg)=Q*segratio(seg);

table pw_brand(j,seg) part-worth of brand attribute
      seg1     seg2    seg3     seg4
A     0.183    1.063  -2.372   -0.203
B     0.832    0.112  -1.589    1.153
C    -0.470   -1.629   1.741   -0.785;

table pw_amps(amps,seg) part-worth of current rating attribute
      seg1     seg2    seg3    seg4
6     1.254    0.453  -1.481  -0.457
9     0.132   -1.422  -0.653  -2.381
12   -1.386    0.969   2.134   2.838;

table pw_life(life,seg) part-worth of product life attribute
      seg1     seg2    seg3    seg4
80   -0.863   -0.127  -4.717   0.802
110   1.337   -0.471  -5.825   0.743
150  -0.474    0.598  10.542  -1.545;

table pw_switch(switch,seg) part-worth of switch attribute
      seg1     seg2    seg3    seg4
PD    0.428    0.299  -3.291  -0.651
TS   -1.015   -0.653  -3.045   0.415
SS    2.392   -0.073   2.463   0.563
TG   -1.805    0.427   3.873  -0.327;

table pw_girth(girth,seg) part-worth of girth attribute
      seg1     seg2    seg3     seg4
S     1.51     0.718   1.512   0.415
L    -1.51    -0.718  -1.512  -0.415;

* price and attributes of competing products
table z(atts,j)
          A      B      C
* price     99     129    79
amps      9      12     6
life      110    150    80
switch    3      1      1
girth     2      1      1;
* Switch type = 1:Paddle/2:Top-Slider/3:Side-Slider/4:Trigger
* Actutator type = 1:Small/2:Large

* calculate utility for the competiting products in the market
* except the utility of price term
vz(seg,j) = pw_brand(j,seg)  !! partworth of brand
     +pw_amps('6',seg)$(z('amps',j)=6)+pw_amps('9',seg)$(z('amps',j)=9)
        +pw_amps('12',seg)$(z('amps',j)=12)
     +pw_life('80',seg)$(z('life',j)=80)+pw_life('110',seg)$(z('life',j)=110)
        +pw_life('150',seg)$(z('life',j)=150)
     +pw_switch('PD',seg)$(z('switch',j)=1)+pw_switch('TS',seg)$(z('switch',j)=2)
        +pw_switch('SS',seg)$(z('switch',j)=3)+pw_switch('TG',seg)$(z('switch',j)=4)
     +pw_girth('S',seg)$(z('girth',j)=1)+pw_girth('L',seg)$(z('girth',j)=2);

variable
   pinew       profit of new product,
   v(seg,j)    utility of product,
   vnew(seg)   utility of new product;
positive variables
   pnew      price of new product,
   p(j)      prices of competing products,
   snew(seg) market share of new product,
   s(seg,j)  market shares of competing products,
   muPU(j)   Lagrangian multpliers for price upper bound,
   muPL(j)   Lagrangian multpliers for price lower bound;

binary variables
   y1(amps),y2(life),y3(switch),y4(girth);

equations
   sum_y1,sum_y2,sum_y3,sum_y4,
   f_profit,f_s(seg,j),f_v(seg,j),
   f_vnew(seg),f_snew(seg),
   f_foccpt(j),f_muGP;

* constraint for binary choices
sum_y1 .. sum(amps  ,y1(amps))  =e= 1;
sum_y2 .. sum(life  ,y2(life))  =e= 1;
sum_y3 .. sum(switch,y3(switch))=e= 1;
sum_y4 .. sum(girth ,y4(girth)) =e= 1;

* calculate logit probability for new product
f_vnew(seg) .. vnew(seg) =e=
      pw_bx(seg)+a2(seg)*pnew*pnew+a1(seg)*pnew+a0(seg)
     +sum(amps,  pw_amps(amps,seg)*y1(amps))
     +sum(life,  pw_life(life,seg)*y2(life))
     +sum(switch,pw_switch(switch,seg)*y3(switch))
     +sum(girth, pw_girth(girth,seg)*y4(girth));

f_v(seg,j) .. v(seg,j) =e= vz(seg,j) + a2(seg)*p(j)*p(j)+a1(seg)*p(j)+a0(seg);

f_snew(seg) .. snew(seg) =e= exp(vnew(seg))/
   ( exp(pw_no(seg))+exp(vnew(seg))+sum(j,exp(v(seg,j))) );

f_s(seg,j) .. s(seg,j) =e= exp(v(seg,j))/
   ( exp(pw_no(seg))+exp(vnew(seg))+sum(jj,exp(v(seg,jj))) );

* FOC equation of price equilibrium for competing products j
f_foccpt(j) .. 0 =e= sum(seg, mktsize(seg)*snew(seg)* (
     (2*a2(seg)*p(j)+a1(seg))*(1-snew(seg))*p(j)+1 ) ) -muPU(j)+muPL(j);

p.lo(j)=75;
p.up(j)=130;
p.l(j)=110;

* here we implement the RegComp(t) method to replace the original KKT
* complemetary slackness conditons
f_muGP ..  -1*sum(jj, muPU(jj)*(p(jj)-p.up(jj))
                     +muPL(jj)*(p.lo(jj)-p(jj)) ) =l= t;

* marginal cost of new product = 1.0324*pnew - 32.758
f_profit ..  pinew =e= (pnew-cnew)*sum(seg, mktsize(seg)*snew(seg) );

muPU.lo(j)=0; muPU.up(j)=inf;
muPL.lo(j)=0; muPL.up(j)=inf;
muPU.l(j)=1;
muPL.l(j)=1;

pnew.lo=75;
pnew.up=130;
pnew.l=100;

y1.l(amps)=0;
y2.l(life)=0;
y3.l(switch)=0;
y4.l(girth)=0;
y1.l('6')=1;
y2.l('80')=1;
y3.l('TS')=1;
y4.l('S')=1;

vnew.lo(seg)=-30;
vnew.up(seg)=30;
vnew.l(seg)=0;
v.lo(seg,j)=-30;
v.up(seg,j)=30;
v.l(seg,j)=0;

model grinder /all/;
*grinder.optfile=1;
option minlp=coinbonmin;
solve grinder using minlp maximizing pinew;

* post-process and display
vcpt(seg,j)=vz(seg,j)+ a2(seg)*p.l(j)*p.l(j)+a1(seg)*p.l(j)+a0(seg);
shrnew(seg)=exp(vnew.l(seg))/(exp(pw_no(seg))+exp(vnew.l(seg))+sum(jj,exp(vcpt(seg,jj))));
shrno(seg)=exp(pw_no(seg))/(exp(pw_no(seg))+exp(vnew.l(seg))+sum(jj,exp(vcpt(seg,jj))));
mktshr(seg,j)=exp(vcpt(seg,j))/(exp(pw_no(seg))+exp(vnew.l(seg))+sum(jj,exp(vcpt(seg,jj))));
profit(j)=sum(seg,mktsize(seg)*mktshr(seg,j))*(p.l(j)-c(j));

file log / '' /;
put log;
put ' ' /;
put 'Optimal design and pricing of angle grinder under price competition' /;
put ' ' /;
put 'Product Price$ Cost $  MktShr% Profit$  Amps  Life  Switch  girth' /;
put '-----------------------------------------------------------------' /;
loop(j,
     put '   ',j.tl:1:0,(p.l(j)):9:1, c(j):8:1,
     (sum(seg,segratio(seg)*mktshr(seg,j))*100):8:2,
     (profit(j)):6:0,'m',z('amps',j):7:0,z('life',j):7:0,z('switch',j):7:0,
     z('girth',j):7:0 /;
     );
put '  New',(pnew.l):8:1, (cnew):8:1,
    (sum(seg,segratio(seg)*shrnew(seg)*100)):8:2, (pinew.l):6:0,'m',
    ( 6$(y1.l('6')=1)+9$(y1.l('9')=1)+12$(y1.l('12')=1)):7:0,
    ( 80$(y2.l('80')=1)+110$(y2.l('110')=1)+150$(y2.l('150')=1)):7:0,
    ( 1$(y3.l('PD')=1)+2$(y3.l('TS')=1)+3$(y3.l('SS')=1)+4$(y3.l('TG')=1) ):7:0,
    ( 1$(y4.l('S')=1)+2$(y4.l('L')=1) ):7:0 /;
put '-----------------------------------------------------------------' /;
put 'Switch type: 1=Paddle 2=Top-Slider 3=Side-Slider 4=Trigger' /;
put ' Girth type: 1=Small  2=Large' /;
put ' ' /;
put 'Mkt Seg',(segratio('seg1')*100):7:2,'%',(segratio('seg2')*100):7:2,'%',
              (segratio('seg3')*100):7:2,'%',(segratio('seg4')*100):7:2,'%',
              (sum(seg,segratio(seg)*100)):7:2,'%' /;
put 'Product  Seg 1   Seg 2   Seg 3   Seg 4   Total%' /;
put '-----------------------------------------------' /;
loop(j,
     put '   ',j.tl:1:0,'  ',(mktshr('seg1',j)*100):8:2,(mktshr('seg2',j)*100):8:2,
     (mktshr('seg3',j)*100):8:2,(mktshr('seg4',j)*100):8:2,
     (sum(seg,segratio(seg)*mktshr(seg,j))*100):8:2 /;
    );
put '  New  ',(shrnew('seg1')*100):7:2,(shrnew('seg2')*100):8:2,
   (shrnew('seg3')*100):8:2,(shrnew('seg4')*100):8:2,
   (sum(seg,segratio(seg)*shrnew(seg)*100)):8:2 /;
put 'Not buy',(shrno('seg1')*100):7:2,(shrno('seg2')*100):8:2,
   (shrno('seg3')*100):8:2,(shrno('seg4')*100):8:2,
   (sum(seg,segratio(seg)*shrno(seg)*100)):8:2 /;
put '-----------------------------------------------' /;
put ' ' /;
putclose;
