% code for class II: optimal design under price equilibrium (foc=0)
%
% Case study from:
% Jeremy J. Michalek, Fred M. Feinberg, and Panos Y. Papalambros
% "Linking Marketing and Engineering Product Design Decisions via
% Analytical Target Cascading" J PROD INNOV MANAG 2005, 22:42, V62
%
% Code: Ching-Shin Norman Shiau
% Date: Feb. 7, 2008
% Design Decisions Laboratory
% Dept. of Mechanical Engineering
% Carnegie Mellon University
% Pittsburgh, PA

format compact
format short
clear all
close all
clc

global P Z Q y c cf J R GP
global LCSegmentSize p_min z_min p_max z_max betaNCLC zcf
load '../latent_class_data.mat'
load 'nash_true.mat'
TolOverall=7e-8;
J=4; % # of existing products in market
brand=['C1 ';'R2 ';'S3 ';'T4 ';'New'];
% #      z1   z2    z3   z4    z5    
Z(1,:)=[350  1.02  120  0.188  1.40]; % C1  
Z(2,:)=[250  0.86  105  0.094  1.25]; % R2 
Z(3,:)=[280  0.89  136  0.156  1.70]; % S3 
Z(4,:)=[320  1.06  115  0.125  1.15]; % T4 
P0=[29.99; 19.99; 25.95; 22.95];

% parameters
Q=5;    % total market size /5/,  !! 5 million units
c=3;    % unit cost (constant) /3/,
cf=1;   % fixed (investment) cost /1/,  !! $1 million
y(1)=0.3;   % Gap between base and cover (inch) /0.30/,
y(2)=0.5;   % Minimum distance between spring and base (inch) /0.50/,
y(3)=1.90;  % Internal thickness of scale (inch) /1.90/,
y(4)=0.25;  % Minimum pinion pitch diameter (inch) /0.25/,
y(5)=3.0;   % Length of window (inch) /3.0/,
y(6)=2.0;   % Width of window (inch) /2.0/,
y(7)=1.13;  % Distance between top of cover and window (inch) /1.13/,
y(8)=1.0;   % Number of lbs measured per tick mark (lbs) /1.0/,
y(9)=1.10;  % Horizontal dist. spring to pivot (inch) /1.10/,
y(10)=0.31; % Length of tick mark gap to number (inch) /0.31/,
y(11)=16.0; % Number of lbs that number spans (lbs) /16.0/,
y(12)=1.29; % Aspect ratio of number length-to-width /1.29/,
y(13)=4.0;  % Min. allow lever dist. base to centerline (in) /4.0/,

% variables
%  x1  Length from Base to Force on Long Lever,
%  x2  Length from Force to Spring on Long Lever,
%  x3  Length from Base to Force on Short Lever,
%  x4  Length from Force to Joint on Short Lever,
%  x5  Length from Force to Joint on Long Lever,
%  x6  Spring Constant,
%  x7  Distance from Base Edge to Spring,
%  x8  Length of Rack,
%  x9  Pitch Diameter of Pinion,
%  x10 Length of Pivot Horizontal Arm,
%  x11 Length of Pivot Vertical Arm,
%  x12 Dial Diameter,
%  x13 Cover Length,
%  x14 Cover Width,
lb=[0.125,0.125,0.125,0.125,0.125,  1,0.5, 1,0.25,0.5,0.5, 9, 9, 9,10];
ub=[   36,   36,   24,   24,   36,200, 12,36,  24,1.9,1.9,13,13,13,30];

options = optimset('Display','off','Algorithm','active-set','TolCon',TolOverall,'TolX',TolOverall);

SolCount=0;

xpall0=[4.6, 7.3, 4.6, 2.3, 1.2,... %x1-5
     90, 0.5, 6.83, 0.35, 0.8, ... %x6-10
     1.7, 9.9, 12.1, 11.6,...  %x11-14
     17, 15, 21, 18, 17]; % pnew/p1-p4

% xpall_old=[3.835  7.945  3.876  2.553  0.183  86.90  0.500...
%      6.830  0.314  0.747  1.846  9.869 12.069 11.600, 15, 15, 15, 15, 15];
xpall_new=zeros(1,15+J);

count=0;
covgflag=0;
disp('round pnew   p1      p2      p3      p4');

% Harker's Relaxation/Diagnolization method
% Each player optimizes his price decision based on the observation of
%  others' in the previous round

xpall=xpall0;
xpall_old=xpall0;
tic;
while(covgflag < 15+J) % check convergence of all variables
    count=count+1;
    disp(sprintf('%-2d    %-0.3f  %-0.3f  %-0.3f  %-0.3f  %-0.3f',count,...
       xpall(15),xpall(16),xpall(17),xpall(18),xpall(19) ));
    % optimization for new product design and price
    [xpnewsol,nfval,exitflag1]=fmincon(@objfunnew,xpall(1:15),...
                 [],[],[],[],lb,ub,@confun,options,xpall);
    if exitflag1 >0
        xpall(1:15)=xpnewsol; % update
    else
        disp(sprintf('warning: exitflag1 = %d',exitflag1)); % not update
    end
    
    for j=1:J, % sequential optimization loop for competitors
        [pjsol,nfvalj,exitflag2]=fmincon(@objfunj,xpall(15+j),...
               [],[],[],[],10,30,[],options,j,xpall);
        % parallel verison: using xpall_old for updating in whole round
        % serial version: using xpall_new for updating right after each
        if exitflag2 > 0 
            xpall(15+j)=pjsol; % update 
        else
            disp(sprintf('warning: exitflag2 = %d',exitflag1)); % not update
        end
    end
    
    covgflag=sum(abs(xpall - xpall_old)<TolOverall*10);
    xpall_old=xpall;

end % end of while loop

cpu_time = toc;
sols=xpall;
psol=xpall(15:19);
zsol=getz(xpall(1:14));
zpsol=[zsol,psol]';
save FOCSOLS.mat sols
% % check Nash condition
disp('===== Start of Nash Verification =====');
VerifyNash

% normaliz first and calculate solution error
zp_min=[200;0.75;100;2/32;0.75;10;10;10;10;10];
zp_max=[400;1.333;140;6/32;1.75;30;30;30;30;30];
norm_inf=max( abs(zpsol-zpsol_nash_true)./(zp_max-zp_min) );
norm_euc=sqrt(sum(( (zpsol-zpsol_nash_true)./(zp_max-zp_min)).^2));
disp(sprintf('norm_inf = %-0.15f',norm_inf));
disp(sprintf('norm_euc = %-0.15f',norm_euc));
disp(sprintf('CPU time = %-0.3f',cpu_time));